@@ -15,7 +15,7 @@ from azdev.utilities import get_azdev_config_dir
 class AzDevCli(CLI):
 
     def get_cli_version(self):
-        from . import __VERSION__
+        from azdev import __VERSION__
         return __VERSION__
 
 
@@ -24,13 +24,13 @@ class AzDevCommandsLoader(CLICommandsLoader):
         from azdev.commands import load_command_table
 
         load_command_table(self, args)
-        return super(AzDevCommandsLoader, self).load_command_table(args)
+        return super().load_command_table(args)
 
     def load_arguments(self, command):
         from azdev.params import load_arguments
 
         load_arguments(self, command)
-        super(AzDevCommandsLoader, self).load_arguments(command)
+        super().load_arguments(command)
 
 
 def main():
